@@ -1,4 +1,4 @@
-// $Id: dbgmainwnd.cpp,v 1.72 2005/01/05 12:41:23 jsixt Exp $
+// $Id: dbgmainwnd.cpp,v 1.73 2005/11/19 22:15:20 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -428,7 +428,7 @@ void DebuggerMainWnd::updateUI()
     // AB: maybe in mainwndbase.cpp?
     actionCollection()->action("file_executable")->setEnabled(m_debugger->isIdle());
     actionCollection()->action("settings_program")->setEnabled(m_debugger->haveExecutable());
-    actionCollection()->action("file_core_dump")->setEnabled(m_debugger->canUseCoreFile());
+    actionCollection()->action("file_core_dump")->setEnabled(m_debugger->canStart());
     actionCollection()->action("exec_step_into")->setEnabled(m_debugger->canSingleStep());
     actionCollection()->action("exec_step_into_by_insn")->setEnabled(m_debugger->canSingleStep());
     actionCollection()->action("exec_step_over")->setEnabled(m_debugger->canSingleStep());
@@ -438,7 +438,7 @@ void DebuggerMainWnd::updateUI()
     actionCollection()->action("exec_movepc")->setEnabled(m_debugger->canSingleStep());
     actionCollection()->action("exec_restart")->setEnabled(m_debugger->canSingleStep());
     actionCollection()->action("exec_attach")->setEnabled(m_debugger->isReady());
-    actionCollection()->action("exec_run")->setEnabled(m_debugger->isReady());
+    actionCollection()->action("exec_run")->setEnabled(m_debugger->canStart() || m_debugger->canSingleStep());
     actionCollection()->action("exec_kill")->setEnabled(m_debugger->haveExecutable() && m_debugger->isProgramActive());
     actionCollection()->action("exec_break")->setEnabled(m_debugger->isProgramRunning());
     actionCollection()->action("exec_arguments")->setEnabled(m_debugger->haveExecutable());
@@ -863,7 +863,7 @@ void DebuggerMainWnd::slotFileExe()
 
 void DebuggerMainWnd::slotFileCore()
 {
-    if (m_debugger->canUseCoreFile())
+    if (m_debugger->canStart())
     {
 	QString corefile = myGetFileName(i18n("Select core dump"),
 					 m_lastDirectory, 0, this);
