@@ -1,4 +1,4 @@
-// $Id: debugger.cpp,v 1.142 2005/01/01 21:23:31 jsixt Exp $
+// $Id: debugger.cpp,v 1.143 2005/11/19 22:15:20 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -523,6 +523,11 @@ bool KDebugger::canChangeBreakpoints()
     return isReady() && !m_programRunning;
 }
 
+bool KDebugger::canStart()
+{
+    return isReady() && !m_programActive;
+}
+
 bool KDebugger::isReady() const 
 {
     return m_haveExecutable &&
