@@ -1,4 +1,4 @@
-// $Id: debugger.h,v 1.86 2005/01/01 21:23:31 jsixt Exp $
+// $Id: debugger.h,v 1.87 2005/11/19 22:15:20 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -240,9 +240,9 @@ public:
     bool canChangeBreakpoints();
 
     /**
-     * Tells whether the debuggee can be changed.
+     * Tells whether a the program is loaded, but not active.
      */
-    bool canUseCoreFile() { return isReady() && !m_programActive; }
+    bool canStart();
 
     /**
      * Add a watch expression.
