@@ -1,4 +1,4 @@
-// $Id: gdbdriver.cpp,v 1.46 2004/08/09 20:15:22 jsixt Exp $
+// $Id: gdbdriver.cpp,v 1.55 2005/11/20 21:31:33 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -72,7 +72,7 @@ static GdbCmdInfo cmds[] = {
     { DCtargetremote, "target remote %s\n", GdbCmdInfo::argString },
     { DCcorefile, "core-file %s\n", GdbCmdInfo::argString },
     { DCattach, "attach %s\n", GdbCmdInfo::argString },
-    { DCinfolinemain, "info line main\n", GdbCmdInfo::argNone },
+    { DCinfolinemain, "kdbg_infolinemain\n", GdbCmdInfo::argNone },
     { DCinfolocals, "kdbg__alllocals\n", GdbCmdInfo::argNone },
     { DCinforegisters, "info all-registers\n", GdbCmdInfo::argNone},
     { DCexamine, "x %s %s\n", GdbCmdInfo::argString2 },
@@ -243,6 +243,13 @@ bool GdbDriver::startup(QString cmdStr)
 	"info locals\n"			/* local vars supersede args with same name */
 	"info args\n"			/* therefore, arguments must come last */
 	"end\n"
+	/*
+	 * Work around a bug in gdb-6.3: "info line main" crashes gdb.
+	 */
+	"define kdbg_infolinemain\n"
+	"list\n"
+	"info line\n"
+	"end\n"
 	// change prompt string and synchronize with gdb
 	"set prompt " PROMPT "\n"
 	;
@@ -1061,19 +1068,38 @@ static bool parseValue(const char*& s, VarTree* variable)
 
 repeat:
     if (*s == '{') {
-	s++;
-	if (!parseNested(s, variable)) {
-	    return false;
+	// Sometimes we find the following output:
+	//  {<text variable, no debug info>} 0x40012000 <access>
+	//  {<data variable, no debug info>}
+	//  {<variable (not text or data), no debug info>}
+	if (strncmp(s, "{<text variable, ", 17) == 0 ||
+	    strncmp(s, "{<data variable, ", 17) == 0 ||
+	    strncmp(s, "{<variable (not text or data), ", 31) == 0)
+	{
+	    const char* start = s;
+	    skipNested(s, '{', '}');
+	    variable->m_value = FROM_LATIN1(start, s-start);
+	    variable->m_value += ' ';	// add only a single space
+	    while (isspace(*s))
+		s++;
+	    goto repeat;
 	}
-	// must be the closing brace
-	if (*s != '}') {
-	    TRACE("parse error: missing } of " +  variable->getText());
-	    return false;
-	}
-	s++;
-	// final white space
-	while (isspace(*s))
+	else
+	{
 	    s++;
+	    if (!parseNested(s, variable)) {
+		return false;
+	    }
+	    // must be the closing brace
+	    if (*s != '}') {
+		TRACE("parse error: missing } of " +  variable->getText());
+		return false;
+	    }
+	    s++;
+	    // final white space
+	    while (isspace(*s))
+		s++;
+	}
     } else {
 	// examples of leaf values (cannot be the empty string):
 	//  123
@@ -1091,6 +1117,8 @@ repeat:
 	//  red
 	//  &parseP (HTMLClueV *, char *)
 	//  Variable "x" is not available.
+	//  The value of variable 'x' is distributed...
+	//  -nan(0xfffff081defa0)
 
 	const char*p = s;
     
@@ -1147,10 +1175,14 @@ repeat:
 	    while (isdigit(*p))
 		p++;
 	    if (*p == '.') {		/* TODO: obey i18n? */
-		// fractional part
-		p++;
-		while (isdigit(*p))
+		// In long arrays an integer may be followed by '...'.
+		// We test for this situation and don't gobble the '...'.
+		if (p[1] != '.' || p[0] != '.') {
+		    // fractional part
 		    p++;
+		    while (isdigit(*p))
+			p++;
+		}
 	    }
 	    if (*p == 'e' || *p == 'E') {
 		p++;
@@ -1188,12 +1220,29 @@ repeat:
 	    if (strncmp(p, "\" is not available.", 19) == 0) {
 		p += 19;
 	    }
+	} else if (strncmp(p, "The value of variable '", 23) == 0) {
+	    p += 23;
+	    skipName(p);
+	    const char* e = strchr(p, '.');
+	    if (e == 0) {
+		p += strlen(p);
+	    } else {
+		p = e+1;
+	    }
 	} else {
 	    // must be an enumeration value
 	    skipName(p);
+	    // hmm, not necessarily: nan (floating point Not a Number)
+	    // is followed by a number in ()
+	    if (*p == '(')
+		skipNested(p, '(', ')');
 	}
 	variable->m_value += FROM_LATIN1(start, p - start);
 
+	// remove line breaks from the value; this is ok since
+	// string values never contain a literal line break
+	variable->m_value.replace('\n', ' ');
+
 	if (checkMultiPart) {
 	    // white space
 	    while (isspace(*p))
@@ -1352,6 +1401,15 @@ static bool parseValueSeq(const char*& s, VarTree* variable)
 	    index++;
 	}
 	variable->appendChild(var);
+	// long arrays may be terminated by '...'
+	if (strncmp(s, "...", 3) == 0) {
+	    s += 3;
+	    VarTree* var = new VarTree("...", VarTree::NKplain);
+	    var->setDeleteChildren(true);
+	    var->m_value = i18n("<additional entries of the array suppressed>");
+	    variable->appendChild(var);
+	    break;
+	}
 	if (*s != ',') {
 	    break;
 	}
@@ -1399,9 +1457,26 @@ static void parseFrameInfo(const char*& s, QString& func,
 	    s++;
 	return;
     }
-    // search opening parenthesis
-    while (*p != '\0' && *p != '(')
-	p++;
+
+    /*
+     * Skip the function name. It is terminated by a left parenthesis
+     * which does not delimit "(anonymous namespace)" and which is
+     * outside the angle brackets <> of template parameter lists.
+     */
+    while (*p != '\0')
+    {
+	if (*p == '<') {
+	    // skip template parameter list
+	    skipNested(p, '<', '>');
+	} else if (*p == '(') {
+	    if (strncmp(p, "(anonymous namespace)", 21) != 0)
+		break;	// parameter list found
+	    p += 21;
+	} else {
+	    p++;
+	}
+    }
+
     if (*p == '\0') {
 	func = start;
 	file = QString();
@@ -2050,12 +2125,17 @@ bool GdbDriver::parseFindType(const char* output, QString& type)
 	return false;
 
     /*
-     * Everything else is the type. We strip off all white-space from the
-     * type.
+     * Everything else is the type. We strip off any leading "const" and any
+     * trailing "&" on the grounds that neither affects the decoding of the
+     * object. We also strip off all white-space from the type.
      */
     output += 7;
+    if (strncmp(output, "const ", 6) == 0)
+        output += 6;
     type = output;
     type.replace(QRegExp("\\s+"), "");
+    if (type.endsWith("&"))
+        type.truncate(type.length() - 1);
     return true;
 }
 
