@@ -1,32 +1,23 @@
-// $Id: mainwndbase.cpp,v 1.33.2.2 2002/03/22 20:33:25 jsixt Exp $
+// $Id: mainwndbase.cpp,v 1.48 2005/01/01 21:23:31 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
 
 #include <kapp.h>
-#if QT_VERSION >= 200
 #include <klocale.h>			/* i18n */
 #include <kconfig.h>
 #include <kmessagebox.h>
-#else
-#include <kmsgbox.h>
-#include <qkeycode.h>
-#endif
-#include <kiconloader.h>
 #include <kstatusbar.h>
-#include <ktoolbar.h>
 #include <kfiledialog.h>
-#include <qpainter.h>
 #include <qtabdialog.h>
-#include <qfileinfo.h>
+#include <qfile.h>
 #include "mainwndbase.h"
 #include "debugger.h"
 #include "gdbdriver.h"
+#include "xsldbgdriver.h"
 #include "prefdebugger.h"
 #include "prefmisc.h"
-#include "procattach.h"
 #include "ttywnd.h"
-#include "updateui.h"
 #include "commandids.h"
 #include "valarray.h"
 #ifdef HAVE_CONFIG
@@ -78,13 +69,7 @@ WatchWindow::~WatchWindow()
 
 bool WatchWindow::eventFilter(QObject*, QEvent* ev)
 {
-    if (ev->type() ==
-#if QT_VERSION < 200
-	Event_KeyPress
-#else
-	QEvent::KeyPress
-#endif
-	)
+    if (ev->type() == QEvent::KeyPress)
     {
 	QKeyEvent* kev = static_cast<QKeyEvent*>(ev);
 	if (kev->key() == Key_Delete) {
@@ -123,7 +108,6 @@ const char defaultHeaderFilter[] = "*.h *.hh *.hpp *.h++";
 
 
 DebuggerMainWndBase::DebuggerMainWndBase() :
-	m_animationCounter(0),
 	m_outputTermCmdStr(defaultTermCmdStr),
 	m_outputTermProc(0),
 	m_ttyLevel(-1),			/* no tty yet */
@@ -138,7 +122,6 @@ DebuggerMainWndBase::DebuggerMainWndBase() :
 	m_debugger(0)
 {
     m_statusActive = i18n("active");
-    m_recentExecList.setAutoDelete(true);
 }
 
 DebuggerMainWndBase::~DebuggerMainWndBase()
@@ -153,31 +136,23 @@ DebuggerMainWndBase::~DebuggerMainWndBase()
     }
 }
 
-void DebuggerMainWndBase::setupDebugger(ExprWnd* localVars,
+void DebuggerMainWndBase::setupDebugger(QWidget* parent,
+					ExprWnd* localVars,
 					ExprWnd* watchVars,
 					QListBox* backtrace)
 {
-    QWidget* parent = dbgMainWnd();
-
-    GdbDriver* driver = new GdbDriver;
-    driver->setLogFileName(m_transcriptFile);
-
-    m_debugger = new KDebugger(parent, localVars, watchVars, backtrace, driver);
+    m_debugger = new KDebugger(parent, localVars, watchVars, backtrace);
 
     QObject::connect(m_debugger, SIGNAL(updateStatusMessage()),
 		     parent, SLOT(slotNewStatusMsg()));
     QObject::connect(m_debugger, SIGNAL(updateUI()),
 		     parent, SLOT(updateUI()));
 
-    QObject::connect(m_debugger, SIGNAL(lineItemsChanged()),
+    QObject::connect(m_debugger, SIGNAL(breakpointsChanged()),
 		     parent, SLOT(updateLineItems()));
     
-    QObject::connect(m_debugger, SIGNAL(animationTimeout()),
-		     parent, SLOT(slotAnimationTimeout()));
     QObject::connect(m_debugger, SIGNAL(debuggerStarting()),
 		     parent, SLOT(slotDebuggerStarting()));
-
-    m_debugger->setDebuggerCmd(m_debuggerCmdStr);
 }
 
 
@@ -197,7 +172,7 @@ void DebuggerMainWndBase::setRemoteDevice(const QString& remoteDevice)
 void DebuggerMainWndBase::setTranscript(const char* name)
 {
     m_transcriptFile = name;
-    if (m_debugger != 0)
+    if (m_debugger != 0 && m_debugger->driver() != 0)
 	m_debugger->driver()->setLogFileName(m_transcriptFile);
 }
 
@@ -211,9 +186,9 @@ const char PopForeground[] = "PopForeground";
 const char BackTimeout[] = "BackTimeout";
 const char TabWidth[] = "TabWidth";
 const char FilesGroup[] = "Files";
-const char RecentExecutables[] = "RecentExecutables";
 const char SourceFileFilter[] = "SourceFileFilter";
 const char HeaderFileFilter[] = "HeaderFileFilter";
+const char GeneralGroup[] = "General";
 
 void DebuggerMainWndBase::saveSettings(KConfig* config)
 {
@@ -233,9 +208,6 @@ void DebuggerMainWndBase::saveSettings(KConfig* config)
     config->writeEntry(TabWidth, m_tabWidth);
     config->writeEntry(SourceFileFilter, m_sourceFilter);
     config->writeEntry(HeaderFileFilter, m_headerFilter);
-    
-    config->setGroup(FilesGroup);
-    config->writeEntry(RecentExecutables, m_recentExecList, ',');
 }
 
 void DebuggerMainWndBase::restoreSettings(KConfig* config)
@@ -262,15 +234,144 @@ void DebuggerMainWndBase::restoreSettings(KConfig* config)
     m_tabWidth = config->readNumEntry(TabWidth, 0);
     m_sourceFilter = config->readEntry(SourceFileFilter, m_sourceFilter);
     m_headerFilter = config->readEntry(HeaderFileFilter, m_headerFilter);
-    
-    config->setGroup(FilesGroup);
-    config->readListEntry(RecentExecutables, m_recentExecList,',');
 }
 
-bool DebuggerMainWndBase::debugProgram(const QString& executable)
+void DebuggerMainWndBase::setAttachPid(const QString& pid)
 {
     assert(m_debugger != 0);
-    return m_debugger->debugProgram(executable);
+    m_debugger->setAttachPid(pid);
+}
+
+bool DebuggerMainWndBase::debugProgram(const QString& executable,
+				       QCString lang, QWidget* parent)
+{
+    assert(m_debugger != 0);
+
+    TRACE(QString().sprintf("trying language '%s'...", lang.data()));
+    DebuggerDriver* driver = driverFromLang(lang);
+
+    if (driver == 0)
+    {
+	// see if there is a language in the per-program config file
+	QString configName = m_debugger->getConfigForExe(executable);
+	if (QFile::exists(configName))
+	{
+	    KSimpleConfig c(configName, true);	// read-only
+	    c.setGroup(GeneralGroup);
+
+	    // Using "GDB" as default here is for backwards compatibility:
+	    // The config file exists but doesn't have an entry,
+	    // so it must have been created by an old version of KDbg
+	    // that had only the GDB driver.
+	    lang = c.readEntry(KDebugger::DriverNameEntry, "GDB").latin1();
+
+	    TRACE(QString().sprintf("...bad, trying config driver %s...",
+			  lang.data()));
+	    driver = driverFromLang(lang);
+	}
+
+    }
+    if (driver == 0)
+    {
+	QCString name = driverNameFromFile(executable);
+
+	TRACE(QString().sprintf("...no luck, trying %s derived"
+				" from file contents", name.data()));
+	driver = driverFromLang(name);
+    }
+    if (driver == 0)
+    {
+	// oops
+	QString msg = i18n("Don't know how to debug language `%1'");
+	KMessageBox::sorry(parent, msg.arg(lang));
+	return false;
+    }
+
+    driver->setLogFileName(m_transcriptFile);
+
+    bool success = m_debugger->debugProgram(executable, driver);
+
+    if (!success)
+    {
+	delete driver;
+
+	QString msg = i18n("Could not start the debugger process.\n"
+			   "Please shut down KDbg and resolve the problem.");
+	KMessageBox::sorry(parent, msg);
+    }
+
+    return success;
+}
+
+// derive driver from language
+DebuggerDriver* DebuggerMainWndBase::driverFromLang(QCString lang)
+{
+    // lang is needed in all lowercase
+    lang = lang.lower();
+
+    // The following table relates languages and debugger drivers
+    static const struct L {
+	const char* shortest;	// abbreviated to this is still unique
+	const char* full;	// full name of language
+	int driver;
+    } langs[] = {
+	{ "c",       "c++",     1 },
+	{ "f",       "fortran", 1 },
+	{ "p",       "python",  3 },
+	{ "x",       "xslt",    2 },
+	// the following are actually driver names
+	{ "gdb",     "gdb",     1 },
+	{ "xsldbg",  "xsldbg",  2 },
+    };
+    const int N = sizeof(langs)/sizeof(langs[0]);
+
+    // lookup the language name
+    int driverID = 0;
+    for (int i = 0; i < N; i++)
+    {
+	const L& l = langs[i];
+
+	// shortest must match
+	if (strncmp(l.shortest, lang, strlen(l.shortest)) != 0)
+	    continue;
+
+	// lang must not be longer than the full name, and it must match
+	if (lang.length() <= strlen(l.full) &&
+	    strncmp(l.full, lang, lang.length()) == 0)
+	{
+	    driverID = l.driver;
+	    break;
+	}
+    }
+    DebuggerDriver* driver = 0;
+    switch (driverID) {
+    case 1:
+	{
+	    GdbDriver* gdb = new GdbDriver;
+	    gdb->setDefaultInvocation(m_debuggerCmdStr);
+	    driver = gdb;
+	}
+	break;
+    case 2:
+	driver = new XsldbgDriver;
+	break;
+    default:
+	// unknown language
+	break;
+    }
+    return driver;
+}
+
+/**
+ * Try to guess the language to use from the contents of the file.
+ */
+QCString DebuggerMainWndBase::driverNameFromFile(const QString& exe)
+{
+    /* Inprecise but simple test to see if file is in XSLT language */
+    if (exe.right(4).lower() == ".xsl")
+	return "XSLT";
+
+    return "GDB";
 }
 
 // helper that gets a file name (it only differs in the caption of the dialog)
@@ -289,198 +390,16 @@ QString DebuggerMainWndBase::myGetFileName(QString caption,
     return filename;
 }
 
-bool DebuggerMainWndBase::handleCommand(int item)
-{
-    /* first commands that don't require the debugger */
-    switch (item) {
-    case ID_FILE_GLOBAL_OPTIONS:
-	doGlobalOptions();
-	return true;
-    }
-
-    // now commands that do
-    if (m_debugger == 0)
-	return false;
-
-    switch (item) {
-    case ID_FILE_EXECUTABLE:
-	if (m_debugger->isIdle())
-	{
-	    // open a new executable
-	    QString executable = myGetFileName(i18n("Select the executable to debug"),
-					     m_lastDirectory, 0, dbgMainWnd());
-	    if (executable.isEmpty())
-		return true;
-
-	    if (debugProgramInteractive(executable)) {
-		addRecentExec(executable);
-	    }
-	}
-	return true;
-    case ID_FILE_PROG_SETTINGS:
-	m_debugger->programSettings(dbgMainWnd());
-	return true;
-    case ID_FILE_COREFILE:
-	if (m_debugger->canUseCoreFile())
-	{
-	    QString corefile = myGetFileName(i18n("Select core dump"),
-					   m_lastDirectory, 0, dbgMainWnd());
-	    if (!corefile.isEmpty()) {
-		m_debugger->useCoreFile(corefile, false);
-	    }
-	}
-	return true;
-    case ID_PROGRAM_RUN:
-	m_debugger->programRun();
-	return true;
-    case ID_PROGRAM_ATTACH:
-	{
-	    ProcAttach dlg(dbgMainWnd());
-	    dlg.setText(m_debugger->attachedPid());
-	    if (dlg.exec()) {
-		m_debugger->attachProgram(dlg.text());
-	    }
-	}
-	return true;
-    case ID_PROGRAM_RUN_AGAIN:
-	m_debugger->programRunAgain();
-	return true;
-    case ID_PROGRAM_STEP:
-	m_debugger->programStep(false);
-	return true;
-    case ID_PROGRAM_STEPI:
-	m_debugger->programStep(true);
-	return true;
-    case ID_PROGRAM_NEXT:
-	m_debugger->programNext(false);
-	return true;
-    case ID_PROGRAM_NEXTI:
-	m_debugger->programNext(true);
-	return true;
-    case ID_PROGRAM_FINISH:
-	m_debugger->programFinish();
-	return true;
-    case ID_PROGRAM_KILL:
-	m_debugger->programKill();
-	return true;
-    case ID_PROGRAM_BREAK:
-	m_debugger->programBreak();
-	return true;
-    case ID_PROGRAM_ARGS:
-	m_debugger->programArgs(dbgMainWnd());
-	return true;
-    }
-    return false;
-}
-
-void DebuggerMainWndBase::updateUIItem(UpdateUI* item)
-{
-    switch (item->id) {
-    case ID_FILE_EXECUTABLE:
-	item->enable(m_debugger->isIdle());
-	break;
-    case ID_FILE_PROG_SETTINGS:
-	item->enable(m_debugger->haveExecutable());
-	break;
-    case ID_FILE_COREFILE:
-	item->enable(m_debugger->canUseCoreFile());
-	break;
-    case ID_PROGRAM_STEP:
-    case ID_PROGRAM_STEPI:
-    case ID_PROGRAM_NEXT:
-    case ID_PROGRAM_NEXTI:
-    case ID_PROGRAM_FINISH:
-    case ID_PROGRAM_UNTIL:
-    case ID_PROGRAM_RUN_AGAIN:
-	item->enable(m_debugger->canSingleStep());
-	break;
-    case ID_PROGRAM_ATTACH:
-    case ID_PROGRAM_RUN:
-	item->enable(m_debugger->isReady());
-	break;
-    case ID_PROGRAM_KILL:
-	item->enable(m_debugger->haveExecutable() && m_debugger->isProgramActive());
-	break;
-    case ID_PROGRAM_BREAK:
-	item->enable(m_debugger->isProgramRunning());
-	break;
-    case ID_PROGRAM_ARGS:
-	item->enable(m_debugger->haveExecutable());
-	break;
-    }
-}
-
-void DebuggerMainWndBase::updateLineItems()
-{
-}
-
-void DebuggerMainWndBase::initAnimation()
-{
-#if QT_VERSION < 200
-    QString path = kapp->kde_datadir() + "/kfm/pics/";
-    QPixmap pixmap;
-    pixmap.load(path + "/kde1.xpm");
-    int numPix = 9;
-#else
-    QPixmap pixmap = BarIcon("kde1");
-    int numPix = 6;
-#endif
-
-    KToolBar* toolbar = dbgToolBar();
-    toolbar->insertButton(pixmap, ID_STATUS_BUSY);
-    toolbar->alignItemRight(ID_STATUS_BUSY, true);
-    
-    // Load animated logo
-    m_animation.setAutoDelete(true);
-    QString n;
-    for (int i = 1; i <= numPix; i++) {
-#if QT_VERSION < 200
-	n.sprintf("/kde%d.xpm", i);
-	QPixmap* p = new QPixmap();
-	p->load(path + n);
-#else
-	n.sprintf("kde%d", i);
-	QPixmap* p = new QPixmap(BarIcon(n));
-#endif
-	if (!p->isNull()) {
-	    m_animation.append(p);
-	} else {
-	    delete p;
-	}
-    }
-    // safeguard: if we did not find a single icon, add a dummy
-    if (m_animation.count() == 0) {
-	QPixmap* pix = new QPixmap(2,2);
-	QPainter p(pix);
-#if QT_VERSION < 200
-	p.fillRect(0,0,2,2,QBrush(white));
-#else
-	p.fillRect(0,0,2,2,QBrush(Qt::white));
-#endif
-	m_animation.append(pix);
-    }
-}
-
-void DebuggerMainWndBase::slotAnimationTimeout()
-{
-    assert(m_animation.count() > 0);	/* must have been initialized */
-    m_animationCounter++;
-    if (m_animationCounter == m_animation.count())
-	m_animationCounter = 0;
-    dbgToolBar()->setButtonPixmap(ID_STATUS_BUSY,
-			       *m_animation.at(m_animationCounter));
-}
-
-void DebuggerMainWndBase::slotNewStatusMsg()
+void DebuggerMainWndBase::newStatusMsg(KStatusBar* statusbar)
 {
     QString msg = m_debugger->statusMessage();
-    dbgStatusBar()->changeItem(msg, ID_STATUS_MSG);
+    statusbar->changeItem(msg, ID_STATUS_MSG);
 }
 
-void DebuggerMainWndBase::doGlobalOptions()
+void DebuggerMainWndBase::doGlobalOptions(QWidget* parent)
 {
-    QTabDialog dlg(dbgMainWnd(), "global_options", true);
-    QString title = kapp->getCaption();
+    QTabDialog dlg(parent, "global_options", true);
+    QString title = kapp->caption();
     title += i18n(": Global options");
     dlg.setCaption(title);
     dlg.setCancelButton(i18n("Cancel"));
@@ -562,15 +481,12 @@ QString DebuggerMainWndBase::createOutputWindow()
 	if (m_outputTermKeepScript.length() != 0) {
 	    fmt = m_outputTermKeepScript.data();
 	}
-#if QT_VERSION < 200
-	QString shellScript(strlen(fmt) + fifoName.length());
-#else
+
 	QString shellScript;
-#endif
 	shellScript.sprintf(fmt, fifoName.data());
 	TRACE("output window script is " + shellScript);
 
-	QString title = kapp->getCaption();
+	QString title = kapp->caption();
 	title += i18n(": Program output");
 
 	// parse the command line specified in the preferences
@@ -707,73 +623,6 @@ void DebuggerMainWndBase::setDebuggerCmdStr(const QString& cmd)
     if (m_debuggerCmdStr == GdbDriver::defaultGdb()) {
 	m_debuggerCmdStr = QString();
     }
-    if (m_debugger != 0) {
-	m_debugger->setDebuggerCmd(m_debuggerCmdStr);
-    }
-}
-
-void DebuggerMainWndBase::addRecentExec(const QString& executable)
-{
-    int pos = m_recentExecList.find(executable);
-    if (pos != 0) {
-	// move to top
-	if (pos > 0)
-	    m_recentExecList.remove(pos);
-	// else entry is new
-
-	// insert on top
-	m_recentExecList.insert(0, executable);
-    } // else pos == 0, which means we dont need to change the list
-
-    // shorten list
-    while (m_recentExecList.count() > MAX_RECENT_FILES) {
-	m_recentExecList.remove(MAX_RECENT_FILES);
-    }
-}
-
-void DebuggerMainWndBase::removeRecentExec(const QString& executable)
-{
-    int pos = m_recentExecList.find(executable);
-    if (pos >= 0) {
-	m_recentExecList.remove(pos);
-    }
-}
-
-bool DebuggerMainWndBase::debugProgramInteractive(const QString& executable)
-{
-    // check the file name
-    QFileInfo fi(executable);
-    m_lastDirectory = fi.dirPath(true);
-
-    if (!fi.isFile()) {
-	QString msgFmt = i18n("`%s' is not a file or does not exist");
-	SIZED_QString(msg, msgFmt.length() + executable.length() + 20);
-#if QT_VERSION < 200
-	msg.sprintf(msgFmt, executable.data());
-	KMsgBox::message(dbgMainWnd(), kapp->appName(),
-			 msg,
-			 KMsgBox::STOP,
-			 i18n("OK"));
-#else
-	msg.sprintf(msgFmt, executable.latin1());
-	KMessageBox::sorry(dbgMainWnd(), msg);
-#endif
-	return false;
-    }
-
-    if (!m_debugger->debugProgram(executable)) {
-	QString msg = i18n("Could not start the debugger process.\n"
-			   "Please shut down KDbg and resolve the problem.");
-#if QT_VERSION < 200
-	KMsgBox::message(dbgMainWnd(), kapp->appName(),
-			 msg,
-			 KMsgBox::STOP,
-			 i18n("OK"));
-#else
-	KMessageBox::sorry(dbgMainWnd(), msg);
-#endif
-    }
-    return true;
 }
 
 
